@@ -27,11 +27,13 @@ class KBlocksView : public QGraphicsView
     void newGame();
     void pauseGame(bool pressed);
     void settingsChanged();
+    void setSoundsEnabled(bool enabled);
     void rotateCW();
     void rotateCCW();
     void moveLeft();
     void moveRight();
     void moveDown();
+    void pushDown();
     
   protected:
     void focusInEvent ( QFocusEvent * event );
