@@ -1,66 +1,61 @@
 /*
- * Permission to use, copy, modify, and distribute this software and its
- * documentation for any purpose and without fee is hereby granted,
- * provided that the above copyright notice appear in all copies and that
- * both that copyright notice and this permission notice appear in
- * supporting documentation.
+ * Copyright (C) 1995 Paul Olav Tvete <paul@troll.no>
+ * Copyright (C) 2000-2009 Stephan Kulow <coolo@kde.org>
+ * Copyright (C) 2009 Parker Coates <parker.coates@gmail.com>
  *
- * This file is provided AS IS with no warranties of any kind.  The author
- * shall have no liability with respect to the infringement of copyrights,
- * trade secrets or any patents by this file or any part thereof.  In no
- * event will the author be liable for any lost revenue or profits or
- * other special, indirect and consequential damages.
-*/
+ * License of original code:
+ * -------------------------------------------------------------------------
+ *   Permission to use, copy, modify, and distribute this software and its
+ *   documentation for any purpose and without fee is hereby granted,
+ *   provided that the above copyright notice appear in all copies and that
+ *   both that copyright notice and this permission notice appear in
+ *   supporting documentation.
+ *
+ *   This file is provided AS IS with no warranties of any kind.  The author
+ *   shall have no liability with respect to the infringement of copyrights,
+ *   trade secrets or any patents by this file or any part thereof.  In no
+ *   event will the author be liable for any lost revenue or profits or
+ *   other special, indirect and consequential damages.
+ * -------------------------------------------------------------------------
+ *
+ * License of modifications/additions made after 2009-01-01:
+ * -------------------------------------------------------------------------
+ *   This program is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU General Public License as
+ *   published by the Free Software Foundation; either version 2 of 
+ *   the License, or (at your option) any later version.
+ *
+ *   This program is distributed in the hope that it will be useful,
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *   GNU General Public License for more details.
+ *
+ *   You should have received a copy of the GNU General Public License
+ *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * -------------------------------------------------------------------------
+ */
 
 #ifndef DEALER_H
 #define DEALER_H
 
 #include "card.h"
-class DealerInfo;
+#include "cardscene.h"
+class CardState;
 class DealerScene;
 class MoveHint;
 #include "pile.h"
 class Solver;
+#include "view.h"
 
 #include <KRandomSequence>
 
-#include <QGraphicsScene>
+#include <QtCore/QSet>
+#include <QtGui/QAction>
+#include <QtGui/QGraphicsScene>
 class QDomDocument;
 
 
 typedef QList<Pile*> PileList;
-
-class DealerInfoList {
-public:
-    static DealerInfoList *self();
-    void add(DealerInfo *);
-
-    const QList<DealerInfo*> games() const { return list; }
-private:
-    QList<DealerInfo*> list;
-    static DealerInfoList *_self;
-    static void cleanupDealerInfoList();
-};
-
-class DealerInfo
-{
-public:
-    DealerInfo(const char *_name, int _index)
-        : name(_name)
-	{
-	    DealerInfoList::self()->add(this);
-            addOldId(_index);
-	}
-    virtual ~DealerInfo(){}
-    const char *name;
-    QList<int> ids;
-    virtual DealerScene *createGame() const = 0;
-    void addOldId(int id) { ids.push_back(id); }
-    bool hasId(int id) { return ids.contains(id); }
-};
-
-class CardState;
-
 typedef QList<CardState> CardStateList;
 
 struct State
@@ -69,92 +64,42 @@ struct State
     QString gameData;
 };
 
-class DealerScene : public QGraphicsScene
+class DealerScene : public CardScene
 {
     Q_OBJECT
 
 public:
+    enum { None = 0, Hint = 1, Demo = 2, Draw = 4, Deal = 8, Redeal = 16 } Actions;
+
     DealerScene();
     ~DealerScene();
 
-    void unmarkAll();
-    void mark(Card *c);
-    QString save_it();
-    Pile * targetPile();
+    virtual void relayoutScene();
 
-protected:
-    virtual void mouseDoubleClickEvent ( QGraphicsSceneMouseEvent * mouseEvent );
-    virtual void mouseMoveEvent ( QGraphicsSceneMouseEvent * mouseEvent );
-    virtual void mousePressEvent ( QGraphicsSceneMouseEvent * mouseEvent );
-    virtual void mouseReleaseEvent ( QGraphicsSceneMouseEvent * mouseEvent );
-    void drawBackground ( QPainter * painter, const QRectF & rect );
-
-public:
-    Pile *findTarget(Card *c);
-    virtual bool cardClicked(Card *);
-    virtual bool pileClicked(Pile *);
-    virtual bool cardDblClicked(Card *);
-
-    bool isMoving(Card *c) const;
-
-    void saveGame(QDomDocument &doc);
-    void openGame(QDomDocument &doc);
-    virtual void mapOldId(int id);
-
-    virtual void getHints();
-    void getSolverHints();
-    void newHint(MoveHint *mh);
-    void clearHints();
-    // it's not const because it changes the random seed
-    virtual MoveHint *chooseHint();
-
-    void setSolverEnabled(bool a);
-    void setAutoDropEnabled(bool a);
-    bool autoDrop() const { return _autodrop; }
+    virtual bool checkPrefering( int checkIndex, const Pile *c1, const CardList& c2) const;
 
     // use this for autodrop times
     int speedUpTime( int delay ) const;
 
+    void createDump( QPaintDevice * );
+
+    void setAutoDropEnabled(bool a);
+    bool autoDropEnabled() const { return _autodrop; }
+
     void setGameNumber(int gmn);
     int gameNumber() const;
 
-    int waiting() const { return _waiting; }
-    void setWaiting(bool w);
-
-    void addPile(Pile *p);
-    void removePile(Pile *p);
-
-    virtual bool isGameLost() const;
-    virtual bool isGameWon() const;
-
     void setGameId(int id);
     int gameId() const;
 
-    virtual void restart() = 0;
-    virtual void stopDemo();
-
-    virtual bool checkRemove( int checkIndex, const Pile *c1, const Card *c) const;
-    virtual bool checkAdd   ( int checkIndex, const Pile *c1, const CardList& c2) const;
-    virtual bool checkPrefering( int checkIndex, const Pile *c1, const CardList& c2) const;
-
-    QPointF maxPilePos() const;
-    qreal offsetX() const;
-    qreal offsetY() const;
-
-    void setSceneSize( const QSize &s );
-
-    void won();
-    bool demoActive() const;
-    int getMoves() const;
-
-    enum { None = 0, Hint = 1, Demo = 2, Draw = 4, Deal = 8, Redeal = 16 } Actions;
-
     void setActions(int actions);
     int actions() const;
 
+    virtual QList<QAction*> configActions() const;
+
+    void setSolverEnabled(bool a);
     void setSolver( Solver *s);
     Solver *solver() const;
-
     void startSolver() const;
     void unlockSolver() const;
     void finishSolver() const;
@@ -162,42 +107,20 @@ public:
     void setNeededFutureMoves(int);
     int neededFutureMoves() const;
 
-    bool isInitialDeal() const;
+    bool demoActive() const;
+    virtual bool isGameLost() const;
+    virtual bool isGameWon() const;
+    bool allowedToStartNewGame();
+    int getMoves() const;
+
+    QString save_it();
+    void saveGame(QDomDocument &doc);
+    void openGame(QDomDocument &doc);
+    virtual void mapOldId(int id);
+    virtual int oldId() const;
     void recordGameStatistics();
 
-    bool cardsAreMoving() const { return !movingCards.empty(); }
-
-    void createDump( QPaintDevice * );
-
-    void updateWonItem();
-
-    bool allowedToStartNewGame();
-
-public slots:
-    virtual bool startAutoDrop();
-    virtual Card *newCards();
-    void hint();
-    void rescale(bool onlypiles);
-
-    State *getState();
-    void setState(State *);
-    void relayoutPiles();
-    void showWonMessage();
-    void takeState();
-    void eraseRedo();
-    void undo();
-    void redo();
-    void slotSolverEnded();
-    void slotSolverFinished();
-    void slotAutoDrop();
-
-    void startNew(int gameNumber = -1);
-
-protected slots:
-    virtual void demo();
-    void waitForDemo(Card *);
-    void waitForWonAnim(Card *c);
-    void toggleDemo();
+    virtual void restart() = 0;
 
 signals:
     void undoPossible(bool poss);
@@ -206,22 +129,46 @@ signals:
     void demoPossible(bool poss);
     void newCardsPossible(bool poss);
 
-    void gameWon(bool withhelp);
     void demoActive(bool en);
     void updateMoves(int moves);
-    void gameLost();
 
-    void gameSolverReset();
-    void gameSolverStart();
-    void gameSolverWon();
-    void gameSolverLost();
-    void gameSolverUnknown();
+    void solverStateChanged(QString text);
 
-private slots:
-    void waitForAutoDrop(Card *);
-    void stopAndRestartSolver();
+public slots:
+    void startNew(int gameNumber = -1);
+    void hint();
+
+    void undo();
+    void redo();
 
 protected:
+    virtual void mouseDoubleClickEvent ( QGraphicsSceneMouseEvent * mouseEvent );
+    virtual void mousePressEvent ( QGraphicsSceneMouseEvent * mouseEvent );
+    virtual void mouseReleaseEvent ( QGraphicsSceneMouseEvent * mouseEvent );
+
+    virtual bool cardDoubleClicked( Card * card );
+
+    virtual void onGameStateAlteredByUser();
+
+    State *getState();
+    void setState(State *);
+    void eraseRedo();
+
+    Pile *findTarget(Card *c);
+
+    QList<MoveHint*> hints() const;
+    virtual void getHints();
+    void getSolverHints();
+    void newHint(MoveHint *mh);
+    void clearHints();
+    // it's not const because it changes the random seed
+    virtual MoveHint *chooseHint();
+
+    virtual void stopDemo();
+
+    void won();
+    void updateWonItem();
+
     // reimplement these to store and load game-specific information in the state structure
     virtual QString getGameState() { return QString(); }
     virtual void setGameState( const QString & ) {}
@@ -231,26 +178,40 @@ protected:
     virtual void setGameOptions( const QString & ) {}
 
     virtual void newDemoMove(Card *m);
-    void considerGameStarted();
 
-    PileList piles;
-    QList<MoveHint*> hints;
-    KRandomSequence randseq;
+    void addCardForDeal( Pile * pile, Card * card, bool faceUp, QPointF startPos );
+    void startDealAnimation();
+
+protected slots:
+    virtual void demo();
+    void waitForDemo(Card *);
+    void toggleDemo();
+
+    void slotSolverEnded();
+    void slotSolverFinished();
+    void startManualDrop();
+
+    void showWonMessage();
+
+    void takeState();
+    virtual Card *newCards();
+    virtual bool drop();
 
 private:
-    QList<QGraphicsItem *> marked;
+    void resetInternals();
 
-    bool moved;
-    CardList movingCards;
-
-    QPointF moving_start;
     bool _autodrop;
     bool _usesolver;
-    int _waiting;
     int gamenumber;
 
     class DealerScenePrivate;
     DealerScenePrivate *d;
+
+    QMap<Card*,QPointF> m_initDealPositions;
+
+private slots:
+    void waitForAutoDrop();
+    void stopAndRestartSolver();
 };
 
 #endif
